<?php

namespace Drupal\workflow_notifications\Form;

use Drupal\Core\Entity\EntityForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\Query\QueryFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\workflow\Entity\Workflow;
use Drupal\user\RoleInterface;


class WorkflowNotificationForm extends EntityForm {
    protected $workflow_id;

  public function __construct(QueryFactory $entity_query) {
    $this->entityQuery = $entity_query;
  }

  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity.query')
    );
  }

  public function form(array $form, FormStateInterface $form_state) {
    $form = parent::form($form, $form_state);

    $workflow_notify = $this->entity;
    
    $roles = array_filter(\Drupal\user\Entity\Role::loadMultiple(), function (RoleInterface $role) {
        return !in_array($role->id(), [
            RoleInterface::ANONYMOUS_ID,
            RoleInterface::AUTHENTICATED_ID,
        ], TRUE);
        });
        $role_options = array_map(function (RoleInterface $role) {
        return $role->label();
        }, $roles);

    $this->workflow_id = \Drupal::request()->get('workflow_type');
    if(isset($this->workflow_id) && !empty($this->workflow_id)) {
        $workflow_type = $this->workflow_id;
    }
    else if(isset($workflow_notify->workflow_type)) {
        $workflow_type = $workflow_notify->workflow_type;
    }
    $workflow = Workflow::load($workflow_type);
    $state_options = array();
    foreach($workflow->getStates() as $entity => $entity_value) {
        $state_options[$entity_value->id()] = $entity_value->label();
    }
    $days = array();
    $days[0] = '-None-';
    for($i=1;$i<=31;$i++){
        $days[$i] = $i . ' Days';
    }
    $form['#attached']['library'][] = 'workflow_notifications/workflow_notifications';
    $form['label'] = array(
            '#type' => 'textfield',
            '#title' => $this->t('Label'),
            '#default_value' => $workflow_notify->label(),
            '#required' => TRUE,
    );
    $form['id'] = array(
      '#type' => 'machine_name',
      '#default_value' => $workflow_notify->id(),
      '#machine_name' => array(
        'exists' => array($this, 'exist'),
      ),
      '#disabled' => !$workflow_notify->isNew(),
    );
    $form['workflow_type'] = array(
        '#type' => 'hidden',
        '#default_value' => isset($workflow_type)? $workflow_type: "",
    );
    $form['from_state'] = array(
            '#type' => 'select',
            '#title' => t('From State'),
            '#options' => $state_options,
            '#default_value' => $workflow_notify->from_state,
            '#required' => TRUE,
        );
        $form['to_state'] = array(
            '#type' => 'select',
            '#title' => t('To State'),
            '#options' => $state_options,
            '#default_value' => $workflow_notify->to_state,
            '#required' => TRUE,
        );
        $form['when_to_trigger'] = array(
            '#type' => 'radios',
            '#title' => t('When To Trigger'),
            '#options' => array('on_state_change' => t('On State Change'), 'before_state_change' => t('Before State Change')),
            '#default_value' => ($workflow_notify->when_to_trigger)? $workflow_notify->when_to_trigger : 'on_state_change',
            '#required' => TRUE,
            '#attributes' => array(
                'class' => array('whn-to-trggr'),
            ),
        );
        $form['days'] = array(
            '#type' => 'select',
            '#title' => t('Days'),
            '#options' => $days,
            '#default_value' => $workflow_notify->days,
            '#attributes' => [
                'class' => ['time'],
            ],
        );
        $form['mail_to'] = array(
            '#type' => 'fieldset',
            '#title' => t('Mail To'),
            '#collapsible' => TRUE,
        );
        $form['mail_to']['roles'] = array(
            '#type' => 'checkboxes',
            '#options' => $role_options,
            '#title' => t('Roles'),
            '#default_value' => $workflow_notify->roles,
        );
        $form['mail_to']['mail_ids'] = array(
            '#type' => 'textarea',
            '#title' => t('MailID'),
            '#default_value' => $workflow_notify->mail_ids,
        );
        $form['template'] = array(
            '#type' => 'fieldset',
            '#title' => t('Template'),
            '#collapsible' => TRUE,
        );
        $form['template']['subject'] = array(
            '#type' => 'textfield',
            '#title' => t('Subject'),
            '#default_value' => $workflow_notify->subject,
            '#required' => TRUE,
        );
        $form['template']['message'] = array(
            '#type' => 'text_format',
            '#title' => t('Message'),
            '#default_value' => $workflow_notify->message['value'],
            '#format' => isset($workflow_notify->message['format'])? $workflow_notify->message['format'] :'basic_html',
            '#required' => TRUE,
        );
        // Token support.
        if (\Drupal::moduleHandler()->moduleExists('token')) {
            $form['tokens'] = [
                '#title' => $this->t('Tokens'),
                '#type' => 'container',
                '#states' => [
                'invisible' => [
                    'input[name="use_token"]' => ['checked' => FALSE],
                ],
                ],
            ];
            $form['tokens']['help'] = [
                '#theme' => 'token_tree_link',
                '#token_types' => ['node', 'workflow_transition', 'workflow_scheduled_transition', 'term', 'site', 'paragraph', 'media', 'comment'],
                '#global_types' => FALSE,
                '#dialog' => TRUE,
            ];
        }
        return $form;
  }
   function validateForm(array &$form, FormStateInterface $form_state) {
        $form_values = $form_state->getValues();
        if($form_values['when_to_trigger'] == 'on_state_change') {
            $form_state->setValue('days',0);
        }
  }
  public function save(array $form, FormStateInterface $form_state) {
    $workflow_notify = $this->entity;
    $status = $workflow_notify->save();

    if ($status) {
      drupal_set_message($this->t('Saved the %label workflow notify.', array(
        '%label' => $workflow_notify->label(),
      )));
    }
    else {
      drupal_set_message($this->t('The %label workflow notify was not saved.', array(
        '%label' => $workflow_notify->label(),
      )));
    }

    $form_state->setRedirect('entity.workflow_notify.collection', ['workflow_type' => $this->workflow_id]);
  }

  public function exist($id) {
    $entity = $this->entityQuery->get('workflow_notify')
      ->condition('id', $id)
      ->execute();
    return (bool) $entity;
  }
}